@@ -12,13 +12,11 @@
 #' @param covariates the matrix of covariates common to every models
 #' @param offsets the matrix of offsets common to every models
 #' @param weights the vector of observation weights
+#' @param control list controlling the optimization and the model
 #' @param formula model formula used for fitting, extracted from the formula in the upper-level call
-#' @param control a list for controlling the optimization. See details.
-#' @param xlevels named listed of factor levels included in the models, extracted from the formula in the upper-level call and used for predictions.
 #' @param var value of the parameter (`rank` for PLNPCA, `sparsity` for PLNnetwork) that identifies the model to be extracted from the collection. If no exact match is found, the model with closest parameter value is returned with a warning.
 #' @param index Integer index of the model to be returned. Only the first value is taken into account.
 #'
-#'
 #' @include PLNfamily-class.R
 #' @importFrom R6 R6Class
 #' @import ggplot2
@@ -40,7 +38,7 @@ PLNPCAfamily <- R6Class(
     ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     ## Creation -----------------------
     #' @description Initialize all models in the collection.
-    initialize = function(ranks, responses, covariates, offsets, weights, formula, xlevels, control) {
+    initialize = function(ranks, responses, covariates, offsets, weights, formula, control) {
       ## initialize the required fields
       super$initialize(responses, covariates, offsets, weights, control)
       private$params <- ranks
@@ -52,7 +50,7 @@ PLNPCAfamily <- R6Class(
 
       ## instantiate as many models as ranks
       self$models <- lapply(ranks, function(rank){
-        model <- PLNPCAfit$new(rank, responses, covariates, offsets, weights, formula, xlevels, control)
+        model <- PLNPCAfit$new(rank, responses, covariates, offsets, weights, formula, control)
         model
       })
     },
@@ -60,17 +58,18 @@ PLNPCAfamily <- R6Class(
     ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     ## Optimization -------------------
     #' @description Call to the C++ optimizer on all models of the collection
-    optimize = function(control) {
+    #' @param config list controlling the optimization.
+    optimize = function(config) {
       self$models <- future.apply::future_lapply(self$models, function(model) {
-        if (control$trace == 1) {
+        if (config$trace == 1) {
           cat("\t Rank approximation =",model$rank, "\r")
           flush.console()
         }
-        if (control$trace > 1) {
+        if (config$trace > 1) {
           cat(" Rank approximation =",model$rank)
           cat("\n\t conservative convex separable approximation for gradient descent")
         }
-        model$optimize(self$responses, self$covariates, self$offsets, self$weights, control)
+        model$optimize(self$responses, self$covariates, self$offsets, self$weights, config)
         model
       }, future.seed = TRUE, future.scheduling = structure(TRUE, ordering = "random"))
     },
