@@ -65,3 +65,16 @@ plot.PLNPCAfit <-
     invisible(p)
   }
 
+#' @describeIn standard_error Component-wise standard errors of B in [`PLNPCAfit`] (not implemented yet)
+#' @export
+standard_error.PLNPCAfit <- function(object, type = c("variational", "jackknife", "sandwich"), parameter = c("B", "Omega")) {
+  par  <- match.arg(parameter)
+  if (par == "Omega")
+    stop("Omega is not estimated as such in PLNPCA models")
+  if (par == "B") {
+    warning("Standard error of B is not implemented yet for PLNPCA models")
+    theta_sd <- coef.PLNfit(object)
+    theta_sd[ , ] <- NA
+    theta_sd
+  }
+}
