@@ -45,3 +45,35 @@ getBestModel <- function(Robject, crit, ...) {UseMethod("getBestModel", Robject)
 #' }
 #' @export
 getModel <- function(Robject, var , index) {UseMethod("getModel"    , Robject)}
+
+#' Display the criteria associated with a collection of PLN fits (a PLNfamily)
+#'
+#' @name plot.PLNfamily
+#'
+#' @param x an R6 object with class [`PLNfamily`]
+#' @param criteria vector of characters. The criteria to plot in c("loglik", "BIC", "ICL").
+#' Default is  c("loglik", "BIC", "ICL").
+#' @param reverse A logical indicating whether to plot the value of the criteria in the "natural" direction
+#' (loglik - 0.5 penalty) or in the "reverse" direction (-2 loglik + penalty). Default to FALSE, i.e use the
+#' natural direction, on the same scale as the log-likelihood.
+#' @param ... additional parameters for S3 compatibility. Not used
+#'
+#' @seealso [plot.PLNPCAfamily()] and [plot.PLNnetworkfamily()]
+#'
+#' @return Produces a plot  representing the evolution of the criteria of the different models considered,
+#' highlighting the best model in terms of BIC and ICL (see details).
+#'
+#' @details The BIC and ICL criteria have the form 'loglik - 1/2 * penalty'
+#' so that they are on the same scale as the model log-likelihood. You can change this direction and use the alternate form '-2*loglik + penalty', as some authors do, by setting \code{reverse = TRUE}.
+#' @examples
+#' data(trichoptera)
+#' trichoptera <- prepare_data(trichoptera$Abundance, trichoptera$Covariate)
+#' myPCAs <- PLNPCA(Abundance ~ 1 + offset(log(Offset)), data = trichoptera, ranks = 1:5)
+#' \dontrun{
+#' plot(myPCAs)
+#' }
+#' @export
+plot.PLNfamily <- function(x, criteria = c("loglik", "BIC", "ICL"), reverse = FALSE, ...) {
+  stopifnot(isPLNfamily(x))
+  x$plot(criteria, reverse)
+}
